@@ -330,6 +330,7 @@ struct aws_credentials_provider *aws_credentials_provider_new_chain_default(
     profile_options.tls_ctx = tls_ctx;
     profile_options.shutdown_options = sub_provider_shutdown_options;
     profile_options.profile_collection_cached = options->profile_collection_cached;
+    profile_options.profile_name_override = options->profile_name_override;
     profile_provider = aws_credentials_provider_new_profile(allocator, &profile_options);
     if (profile_provider != NULL) {
         providers[index++] = profile_provider;
@@ -343,6 +344,7 @@ struct aws_credentials_provider *aws_credentials_provider_new_chain_default(
     sts_options.tls_ctx = tls_ctx;
     sts_options.shutdown_options = sub_provider_shutdown_options;
     sts_options.config_profile_collection_cached = options->profile_collection_cached;
+    sts_options.profile_name_override = options->profile_name_override;
     sts_provider = aws_credentials_provider_new_sts_web_identity(allocator, &sts_options);
     if (sts_provider != NULL) {
         providers[index++] = sts_provider;
@@ -354,6 +356,7 @@ struct aws_credentials_provider *aws_credentials_provider_new_chain_default(
     AWS_ZERO_STRUCT(process_options);
     process_options.shutdown_options = sub_provider_shutdown_options;
     process_options.config_profile_collection_cached = options->profile_collection_cached;
+    process_options.profile_to_use = options->profile_name_override;
     process_provider = aws_credentials_provider_new_process(allocator, &process_options);
     if (process_provider != NULL) {
         providers[index++] = process_provider;
