@@ -36,11 +36,23 @@ struct der_tlv {
     uint8_t *value;
 };
 
-static void s_decode_tlv(struct der_tlv *tlv) {
+static int s_decode_tlv(struct der_tlv *tlv) {
     if (tlv->tag == AWS_DER_INTEGER) {
+        if (tlv->length == 0) {
+            return aws_raise_error(AWS_ERROR_CAL_MALFORMED_ASN1_ENCOUNTERED);
+        }
+
         uint8_t first_byte = tlv->value[0];
-        /* if the first byte is 0, it just denotes unsigned and should be removed */
-        if (first_byte == 0x00) {
+        if (first_byte & 0x80) {
+            return aws_raise_error(AWS_ERROR_CAL_DER_UNSUPPORTED_NEGATIVE_INT);
+        }
+
+        /* if its multibyte int and first byte is 0, strip it since it was added
+         * to indicate to der that it is positive number.
+         * if len is 1 and first byte is 0, then the number is just zero, so
+         * leave it as is.
+         */
+        if (tlv->length > 1 && first_byte == 0x00) {
             tlv->length -= 1;
             tlv->value += 1;
         }
@@ -49,6 +61,8 @@ static void s_decode_tlv(struct der_tlv *tlv) {
         tlv->length -= 1;
         tlv->value += 1;
     }
+
+    return AWS_OP_SUCCESS;
 }
 
 static int s_der_read_tlv(struct aws_byte_cursor *cur, struct der_tlv *tlv) {
@@ -88,10 +102,16 @@ static int s_der_read_tlv(struct aws_byte_cursor *cur, struct der_tlv *tlv) {
         len = len_bytes;
     }
 
+    if (len > cur->len) {
+        return aws_raise_error(AWS_ERROR_CAL_MALFORMED_ASN1_ENCOUNTERED);
+    }
+
     tlv->tag = tag;
     tlv->length = len;
     tlv->value = (tag == AWS_DER_NULL) ? NULL : cur->ptr;
-    s_decode_tlv(tlv);
+    if (s_decode_tlv(tlv)) {
+        return AWS_OP_ERR;
+    }
     aws_byte_cursor_advance(cur, len);
 
     return AWS_OP_SUCCESS;
@@ -222,7 +242,7 @@ void aws_der_encoder_destroy(struct aws_der_encoder *encoder) {
     aws_mem_release(encoder->allocator, encoder);
 }
 
-int aws_der_encoder_write_integer(struct aws_der_encoder *encoder, struct aws_byte_cursor integer) {
+int aws_der_encoder_write_unsigned_integer(struct aws_der_encoder *encoder, struct aws_byte_cursor integer) {
     AWS_FATAL_ASSERT(integer.len <= UINT32_MAX);
     struct der_tlv tlv = {
         .tag = AWS_DER_INTEGER,
@@ -391,12 +411,13 @@ int s_parse_cursor(struct aws_der_decoder *decoder, struct aws_byte_cursor cur)
     while (cur.len) {
         struct der_tlv tlv = {0};
         if (s_der_read_tlv(&cur, &tlv)) {
-            return aws_raise_error(AWS_ERROR_CAL_MALFORMED_ASN1_ENCOUNTERED);
+            return AWS_OP_ERR;
         }
         /* skip trailing newlines in the stream after any TLV */
         while (cur.len && *cur.ptr == '\n') {
             aws_byte_cursor_advance(&cur, 1);
         }
+
         if (aws_array_list_push_back(&decoder->tlvs, &tlv)) {
             return aws_raise_error(AWS_ERROR_INVALID_STATE);
         }
@@ -472,7 +493,7 @@ int aws_der_decoder_tlv_string(struct aws_der_decoder *decoder, struct aws_byte_
     return AWS_OP_SUCCESS;
 }
 
-int aws_der_decoder_tlv_integer(struct aws_der_decoder *decoder, struct aws_byte_cursor *integer) {
+int aws_der_decoder_tlv_unsigned_integer(struct aws_der_decoder *decoder, struct aws_byte_cursor *integer) {
     struct der_tlv tlv = s_decoder_tlv(decoder);
     if (tlv.tag != AWS_DER_INTEGER) {
         return aws_raise_error(AWS_ERROR_CAL_MISMATCHED_DER_TYPE);
