@@ -17,6 +17,7 @@
 struct exponential_backoff_strategy {
     struct aws_retry_strategy base;
     struct aws_exponential_backoff_retry_options config;
+    struct aws_shutdown_callback_options shutdown_options;
 };
 
 struct exponential_backoff_retry_token {
@@ -25,10 +26,13 @@ struct exponential_backoff_retry_token {
     struct aws_atomic_var last_backoff;
     size_t max_retries;
     uint64_t backoff_scale_factor_ns;
+    uint64_t maximum_backoff_ns;
     enum aws_exponential_backoff_jitter_mode jitter_mode;
     /* Let's not make this worse by constantly moving across threads if we can help it */
     struct aws_event_loop *bound_loop;
     uint64_t (*generate_random)(void);
+    aws_generate_random_fn *generate_random_impl;
+    void *generate_random_user_data;
     struct aws_task retry_task;
 
     struct {
@@ -43,7 +47,14 @@ static void s_exponential_retry_destroy(struct aws_retry_strategy *retry_strateg
     if (retry_strategy) {
         struct exponential_backoff_strategy *exponential_strategy = retry_strategy->impl;
         struct aws_event_loop_group *el_group = exponential_strategy->config.el_group;
+        aws_simple_completion_callback *completion_callback =
+            exponential_strategy->shutdown_options.shutdown_callback_fn;
+        void *completion_user_data = exponential_strategy->shutdown_options.shutdown_callback_user_data;
+
         aws_mem_release(retry_strategy->allocator, exponential_strategy);
+        if (completion_callback != NULL) {
+            completion_callback(completion_user_data);
+        }
         aws_ref_count_release(&el_group->ref_count);
     }
 }
@@ -129,8 +140,13 @@ static int s_exponential_retry_acquire_token(
     backoff_retry_token->max_retries = exponential_backoff_strategy->config.max_retries;
     backoff_retry_token->backoff_scale_factor_ns = aws_timestamp_convert(
         exponential_backoff_strategy->config.backoff_scale_factor_ms, AWS_TIMESTAMP_MILLIS, AWS_TIMESTAMP_NANOS, NULL);
+    backoff_retry_token->maximum_backoff_ns = aws_timestamp_convert(
+        exponential_backoff_strategy->config.max_backoff_secs, AWS_TIMESTAMP_SECS, AWS_TIMESTAMP_NANOS, NULL);
     backoff_retry_token->jitter_mode = exponential_backoff_strategy->config.jitter_mode;
     backoff_retry_token->generate_random = exponential_backoff_strategy->config.generate_random;
+    backoff_retry_token->generate_random_impl = exponential_backoff_strategy->config.generate_random_impl;
+    backoff_retry_token->generate_random_user_data = exponential_backoff_strategy->config.generate_random_user_data;
+
     aws_atomic_init_int(&backoff_retry_token->current_retry_count, 0);
     aws_atomic_init_int(&backoff_retry_token->last_backoff, 0);
 
@@ -158,8 +174,12 @@ static inline uint64_t s_random_in_range(uint64_t from, uint64_t to, struct expo
     if (!diff) {
         return 0;
     }
-
-    uint64_t random = token->generate_random();
+    uint64_t random;
+    if (token->generate_random_impl) {
+        random = token->generate_random_impl(token->generate_random_user_data);
+    } else {
+        random = token->generate_random();
+    }
     return min + random % (diff);
 }
 
@@ -167,7 +187,8 @@ typedef uint64_t(compute_backoff_fn)(struct exponential_backoff_retry_token *tok
 
 static uint64_t s_compute_no_jitter(struct exponential_backoff_retry_token *token) {
     uint64_t retry_count = aws_min_u64(aws_atomic_load_int(&token->current_retry_count), 63);
-    return aws_mul_u64_saturating((uint64_t)1 << retry_count, token->backoff_scale_factor_ns);
+    uint64_t backoff_ns = aws_mul_u64_saturating((uint64_t)1 << retry_count, token->backoff_scale_factor_ns);
+    return aws_min_u64(backoff_ns, token->maximum_backoff_ns);
 }
 
 static uint64_t s_compute_full_jitter(struct exponential_backoff_retry_token *token) {
@@ -181,8 +202,8 @@ static uint64_t s_compute_deccorelated_jitter(struct exponential_backoff_retry_t
     if (!last_backoff_val) {
         return s_compute_full_jitter(token);
     }
-
-    return s_random_in_range(token->backoff_scale_factor_ns, aws_mul_u64_saturating(last_backoff_val, 3), token);
+    uint64_t backoff_ns = aws_min_u64(token->maximum_backoff_ns, aws_mul_u64_saturating(last_backoff_val, 3));
+    return s_random_in_range(token->backoff_scale_factor_ns, backoff_ns, token);
 }
 
 static compute_backoff_fn *s_backoff_compute_table[] = {
@@ -266,7 +287,7 @@ static int s_exponential_retry_schedule_retry(
             AWS_LS_IO_EXPONENTIAL_BACKOFF_RETRY_STRATEGY,
             "id=%p: retry token %p is already scheduled.",
             (void *)backoff_retry_token->base.retry_strategy,
-            (void *)token)
+            (void *)token);
         return aws_raise_error(AWS_ERROR_INVALID_STATE);
     }
 
@@ -297,7 +318,8 @@ static struct aws_retry_strategy_vtable s_exponential_retry_vtable = {
     .release_token = s_exponential_backoff_release_token,
 };
 
-static uint64_t s_default_gen_rand(void) {
+static uint64_t s_default_gen_rand(void *user_data) {
+    (void)user_data;
     uint64_t res = 0;
     aws_device_random_u64(&res);
     return res;
@@ -307,9 +329,9 @@ struct aws_retry_strategy *aws_retry_strategy_new_exponential_backoff(
     struct aws_allocator *allocator,
     const struct aws_exponential_backoff_retry_options *config) {
     AWS_PRECONDITION(config);
-    AWS_PRECONDITION(config->el_group)
-    AWS_PRECONDITION(config->jitter_mode <= AWS_EXPONENTIAL_BACKOFF_JITTER_DECORRELATED)
-    AWS_PRECONDITION(config->max_retries)
+    AWS_PRECONDITION(config->el_group);
+    AWS_PRECONDITION(config->jitter_mode <= AWS_EXPONENTIAL_BACKOFF_JITTER_DECORRELATED);
+    AWS_PRECONDITION(config->max_retries);
 
     if (config->max_retries > 63 || !config->el_group ||
         config->jitter_mode > AWS_EXPONENTIAL_BACKOFF_JITTER_DECORRELATED) {
@@ -341,8 +363,9 @@ struct aws_retry_strategy *aws_retry_strategy_new_exponential_backoff(
     exponential_backoff_strategy->config.el_group =
         aws_ref_count_acquire(&exponential_backoff_strategy->config.el_group->ref_count);
 
-    if (!exponential_backoff_strategy->config.generate_random) {
-        exponential_backoff_strategy->config.generate_random = s_default_gen_rand;
+    if (!exponential_backoff_strategy->config.generate_random &&
+        !exponential_backoff_strategy->config.generate_random_impl) {
+        exponential_backoff_strategy->config.generate_random_impl = s_default_gen_rand;
     }
 
     if (!exponential_backoff_strategy->config.max_retries) {
@@ -353,5 +376,12 @@ struct aws_retry_strategy *aws_retry_strategy_new_exponential_backoff(
         exponential_backoff_strategy->config.backoff_scale_factor_ms = 25;
     }
 
+    if (!exponential_backoff_strategy->config.max_backoff_secs) {
+        exponential_backoff_strategy->config.max_backoff_secs = 20;
+    }
+
+    if (config->shutdown_options) {
+        exponential_backoff_strategy->shutdown_options = *config->shutdown_options;
+    }
     return &exponential_backoff_strategy->base;
 }
