@@ -66,6 +66,27 @@ typedef void(aws_mqtt_client_on_connection_complete_fn)(
     bool session_present,
     void *userdata);
 
+/* Called when a connection attempt succeed (with a successful CONNACK)
+ *
+ * The callback is derived from aws_mqtt_client_on_connection_complete_fn.
+ * It gets triggered when connection succeed (with a successful CONNACK)
+ */
+typedef void(aws_mqtt_client_on_connection_success_fn)(
+    struct aws_mqtt_client_connection *connection,
+    enum aws_mqtt_connect_return_code return_code,
+    bool session_present,
+    void *userdata);
+
+/* Called if the connection attempt failed.
+ *
+ * The callback is derived from aws_mqtt_client_on_connection_complete_fn.
+ * It gets triggered when connection failed.
+ */
+typedef void(aws_mqtt_client_on_connection_failure_fn)(
+    struct aws_mqtt_client_connection *connection,
+    int error_code,
+    void *userdata);
+
 /* Called if the connection to the server is lost. */
 typedef void(aws_mqtt_client_on_connection_interrupted_fn)(
     struct aws_mqtt_client_connection *connection,
@@ -140,6 +161,11 @@ typedef void(aws_mqtt_client_publish_received_fn)(
 /** Called when a connection is closed, right before any resources are deleted */
 typedef void(aws_mqtt_client_on_disconnect_fn)(struct aws_mqtt_client_connection *connection, void *userdata);
 
+/**
+ * Signature of callback invoked on a connection destruction.
+ */
+typedef void(aws_mqtt_client_on_connection_termination_fn)(void *userdata);
+
 /**
  * Function to invoke when the websocket handshake request transformation completes.
  * This function MUST be invoked or the application will soft-lock.
@@ -422,6 +448,23 @@ int aws_mqtt_client_connection_set_reconnect_timeout(
     uint64_t min_timeout,
     uint64_t max_timeout);
 
+/**
+ * Sets the callbacks to call when a connection succeeds or fails
+ *
+ * \param[in] connection                The connection object
+ * \param[in] on_connection_success     The function to call when a connection is successful or gets resumed
+ * \param[in] on_connection_success_ud  Userdata for on_connection_success
+ * \param[in] on_connection_failure     The function to call when a connection fails
+ * \param[in] on_connection_failure_ud  Userdata for on_connection_failure
+ */
+AWS_MQTT_API
+int aws_mqtt_client_connection_set_connection_result_handlers(
+    struct aws_mqtt_client_connection *connection,
+    aws_mqtt_client_on_connection_success_fn *on_connection_success,
+    void *on_connection_success_ud,
+    aws_mqtt_client_on_connection_failure_fn *on_connection_failure,
+    void *on_connection_failure_ud);
+
 /**
  * Sets the callbacks to call when a connection is interrupted and resumed.
  *
@@ -468,6 +511,19 @@ int aws_mqtt_client_connection_set_on_any_publish_handler(
     aws_mqtt_client_publish_received_fn *on_any_publish,
     void *on_any_publish_ud);
 
+/**
+ * Sets the callback to call on a connection destruction.
+ *
+ * \param[in] connection        The connection object.
+ * \param[in] on_termination    The function to call when a connection is destroyed.
+ * \param[in] on_termination_ud Userdata for on_termination.
+ */
+AWS_MQTT_API
+int aws_mqtt_client_connection_set_connection_termination_handler(
+    struct aws_mqtt_client_connection *connection,
+    aws_mqtt_client_on_connection_termination_fn *on_termination,
+    void *on_termination_ud);
+
 /**
  * Opens the actual connection defined by aws_mqtt_client_connection_new.
  * Once the connection is opened, on_connack will be called. Only called when connection is disconnected.
