@@ -98,6 +98,10 @@ const struct aws_s3_platform_info *aws_s3_get_current_platform_info(void) {
     return aws_s3_get_platform_info_for_current_environment(s_loader);
 }
 
+struct aws_array_list aws_s3_get_platforms_with_recommended_config(void) {
+    return aws_s3_get_recommended_platforms(s_loader);
+}
+
 void aws_s3_library_clean_up(void) {
     if (!s_library_initialized) {
         return;
