@@ -3,7 +3,7 @@
 
 import awscrt.exceptions
 from awscrt.http import HttpClientConnection, HttpClientStream, HttpHeaders, HttpProxyOptions, HttpRequest, HttpVersion
-from awscrt.io import ClientBootstrap, ClientTlsContext, DefaultHostResolver, EventLoopGroup, TlsConnectionOptions, TlsContextOptions
+from awscrt.io import ClientBootstrap, ClientTlsContext, DefaultHostResolver, EventLoopGroup, TlsConnectionOptions, TlsContextOptions, TlsCipherPref
 from concurrent.futures import Future
 from http.server import HTTPServer, SimpleHTTPRequestHandler
 from io import BytesIO
@@ -57,10 +57,9 @@ class TestClient(NativeResourceTest):
 
         self.server = HTTPServer((self.hostname, 0), TestRequestHandler)
         if secure:
-            self.server.socket = ssl.wrap_socket(self.server.socket,
-                                                 keyfile="test/resources/unittest.key",
-                                                 certfile='test/resources/unittest.crt',
-                                                 server_side=True)
+            context = ssl.SSLContext()
+            context.load_cert_chain(certfile='test/resources/unittest.crt', keyfile="test/resources/unittest.key")
+            self.server.socket = context.wrap_socket(self.server.socket, server_side=True)
         self.port = self.server.server_address[1]
 
         # put requests are stored in this dict
@@ -74,9 +73,10 @@ class TestClient(NativeResourceTest):
         self.server.server_close()
         self.server_thread.join()
 
-    def _new_client_connection(self, secure, proxy_options=None):
+    def _new_client_connection(self, secure, proxy_options=None, cipher_pref=TlsCipherPref.DEFAULT):
         if secure:
             tls_ctx_opt = TlsContextOptions()
+            tls_ctx_opt.cipher_pref = cipher_pref
             tls_ctx_opt.verify_peer = False
             tls_ctx = ClientTlsContext(tls_ctx_opt)
             tls_conn_opt = tls_ctx.new_connection_options()
@@ -94,10 +94,10 @@ class TestClient(NativeResourceTest):
                                                      proxy_options=proxy_options)
         return connection_future.result(self.timeout)
 
-    def _test_connect(self, secure):
+    def _test_connect(self, secure, cipher_pref=TlsCipherPref.DEFAULT):
         self._start_server(secure)
         try:
-            connection = self._new_client_connection(secure)
+            connection = self._new_client_connection(secure, cipher_pref=cipher_pref)
 
             # close connection
             shutdown_error_from_close_future = connection.close().exception(self.timeout)
@@ -368,6 +368,10 @@ class TestClient(NativeResourceTest):
 
         self.assertEqual(None, connection.close().exception(self.timeout))
 
+    @unittest.skipIf(not TlsCipherPref.PQ_TLSv1_0_2021_05.is_supported(), "Cipher pref not supported")
+    def test_connect_pq_tlsv1_0_2021_05(self):
+        self._test_connect(secure=True, cipher_pref=TlsCipherPref.PQ_TLSv1_0_2021_05)
+
 
 if __name__ == '__main__':
     unittest.main()
