@@ -20,8 +20,10 @@ from awscrt.s3 import (
     S3ChecksumLocation,
     S3Client,
     S3RequestType,
+    S3ResponseError,
     CrossProcessLock,
     create_default_s3_signing_config,
+    get_optimized_platforms,
 )
 from awscrt.io import (
     ClientBootstrap,
@@ -215,6 +217,11 @@ class S3ClientTest(NativeResourceTest):
         del s3_client
         self.assertTrue(shutdown_event.wait(self.timeout))
 
+    def test_get_optimized_platforms(self):
+        platform_list = get_optimized_platforms()
+        self.assertTrue(len(platform_list) > 0)
+        self.assertTrue("p4d.24xlarge" in platform_list)
+
 
 @unittest.skipUnless(os.environ.get('AWS_TEST_S3'), 'set env var to run test: AWS_TEST_S3')
 class S3RequestTest(NativeResourceTest):
@@ -288,6 +295,8 @@ class S3RequestTest(NativeResourceTest):
         self.assertEqual(self.done_status_code, self.response_status_code,
                          "status-code from on_done doesn't match code from on_headers")
         self.assertIsNone(self.done_error)
+        self.assertIsNone(self.done_error_headers)
+        self.assertIsNone(self.done_error_body)
         headers = HttpHeaders(self.response_headers)
         self.assertIsNone(headers.get("Content-Range"))
         body_length = headers.get("Content-Length")
@@ -596,15 +605,64 @@ class S3RequestTest(NativeResourceTest):
         self._test_s3_put_get_object(request, S3RequestType.PUT_OBJECT, "AWS_ERROR_S3_INVALID_RESPONSE_STATUS")
 
         # check that data from on_done callback came through correctly
-        self.assertIsNotNone(self.done_error)
+        self.assertIsInstance(self.done_error, S3ResponseError)
         self.assertEqual(self.done_status_code, 400)
+        self.assertEqual(self.done_error.status_code, 400)
         self.assertIsNotNone(self.done_error_headers)
         self.assertTrue(any(h[0].lower() == 'x-amz-request-id' for h in self.done_error_headers))
+        self.assertListEqual(self.done_error_headers, self.done_error.headers)
         self.assertIsNotNone(self.done_error_body)
         self.assertTrue(b"InvalidDigest" in self.done_error_body)
+        self.assertEqual(self.done_error_body, self.done_error.body)
 
         put_body_stream.close()
 
+    def test_on_headers_callback_failure(self):
+        def _explode(**kwargs):
+            raise RuntimeError("Error in on_headers callback")
+
+        request = self._get_object_request(self.get_test_object_path)
+        s3_client = s3_client_new(False, self.region, 5 * MB)
+        s3_request = s3_client.make_request(
+            request=request,
+            type=S3RequestType.GET_OBJECT,
+            on_headers=_explode,
+            on_body=self._on_request_body,
+        )
+
+        finished_future = s3_request.finished_future
+        shutdown_event = s3_request.shutdown_event
+        s3_request = None
+        self.assertTrue(shutdown_event.wait(self.timeout))
+
+        e = finished_future.exception()
+        # check that data from on_done callback came through correctly
+        self.assertIsInstance(e, RuntimeError)
+        self.assertEqual(str(e), "Error in on_headers callback")
+
+    def test_on_body_callback_failure(self):
+        def _explode(**kwargs):
+            raise RuntimeError("Error in on_body callback")
+
+        request = self._get_object_request(self.get_test_object_path)
+        s3_client = s3_client_new(False, self.region, 5 * MB)
+        s3_request = s3_client.make_request(
+            request=request,
+            type=S3RequestType.GET_OBJECT,
+            on_headers=self._on_request_headers,
+            on_body=_explode,
+        )
+
+        finished_future = s3_request.finished_future
+        shutdown_event = s3_request.shutdown_event
+        s3_request = None
+        self.assertTrue(shutdown_event.wait(self.timeout))
+
+        e = finished_future.exception()
+        # check that data from on_done callback came through correctly
+        self.assertIsInstance(e, RuntimeError)
+        self.assertEqual(str(e), "Error in on_body callback")
+
     def test_special_filepath_upload(self):
         # remove the input file when request done
         content_length = 10 * MB
